#include "CyanBusCRC.hpp"

// C++ system level
// #include <functional>

// ESP32 specific
#include "esp_log.h"
#include "esp32/rom/crc.h"

// project specific
#include "SpiderLib/NumberTypes.hpp"


static const char* TAG = "CyanBus";

namespace SiliconTorch {

  namespace Impl {


    // Checks CyanBus CRC
    // Note: The last 4 bytes of buffer contain the CRC to check against and are included in the given length
    bool checkCRC(u8* buffer, u32 length) {

      if (length < 4) {
        ESP_LOGD(TAG, "CRC calculation failed: Input buffer too small! Got bytes[ %d ] but needs bytes[ >= 4 ]", length);
        return false;
      }

      // u32 _init = 0x00000000;
      u32 _init = 0xFFFFFFFF;
      u32 calculatedCRC = crc32_be(_init, buffer, length - 4);
      u32 packetCRC = reinterpret_cast<u32*>(&buffer[length - 4])[0];

      bool crcOK = packetCRC == calculatedCRC;

      ESP_LOGD("CheckCRC", "crc⏻[ %s ]   packetCRC = 0x%X   calcCRC = 0x%X", crcOK ? "✔" : "✘", packetCRC, calculatedCRC);

      return crcOK;
    }


  }
}
