#pragma once

// C++ system level
#include <set>
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
#include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>

// qthing stuff
// #include <qthing>


namespace SiliconTorch {

  namespace ServiceManager {

    extern const str NVSStateKey;

    enum NVSState : u8 {
      Disabled = 0,
      Enabled  = 1,
      Invalid
    };

    class Service;

    using ServiceSet    = std::set<Service*>;
    using ServiceLookup = std::function<Service*(const str&)>;


    class Service {
      public:

        Service();


        // ╔ To be implemented by user! ╗

        virtual void init()  = 0;
        virtual void start() = 0;

        // ╚════════════════════════════╝


        const str& getName() const;
        const str& getNameSpace() const;

        bool isEnabled() const;
        void setEnabled(bool enabled);

        Service* otherByName(const str& name);

        // internal stuff, only to be called by ServiceManager!
        void preInit(ServiceLookup lookup);
        void postInit();

        ServiceSet getWantedServices();
        ServiceSet getRequiredServices();

      protected:

        void setName(const str& name);
        void setNameSpace(const str& nameSpace);

        // Service will be started after the wanted one (if wanted is disabled this limitation is waived)    // TODO: (waived / raised / lowered?)
        void addWantedService(const str& name);

        // Service will only be started after the required one
        void addRequiredService(const str& name);

      private:

        NVSState nvsState = Disabled;

        bool hasErrors = false;  // Set if an error is detected. Prevents the service from being started

        bool attrsLocked = false;
        str name = "";
        str nameSpace = "";

        std::set<std::string>* wantedNames   = new std::set<std::string>();
        std::set<std::string>* requiredNames = new std::set<std::string>();

        ServiceSet wantedServices;
        ServiceSet requiredServices;

        ServiceLookup serviceLookup = [](const str& ignored) { return (Service*)NULL; };

    };


  }
}
