#include "SNTP.hpp"

// ESP32 specific
#include "esp_log.h"
#include "esp_sntp.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

// #include "driver/ledc.h"

// C++ system level
#include <ctime>
#include <cstring>
#include <functional>

// project specific
#include "SpiderLib/NumberTypes.hpp"


/*extern "C" {
  void setenv(char* env, char* value, int overwrite);
  void tzset();
}*/


static const char* TAG = "SNTP";


static void ntpTask(void* evHdl) {

  std::function<void()>* syncEventHandler = (std::function<void()>*) evHdl;

  time_t now = 0;
  struct tm timeinfo;
  memset(&timeinfo, 0, sizeof(tm));

  do {
    vTaskDelay(100 / portTICK_PERIOD_MS);
    time(&now);
    localtime_r(&now, &timeinfo);
  } while (timeinfo.tm_year < 100);


  ESP_LOGI(TAG, "Time synchronized: %02d:%02d:%02d %d-%02d-%02d", timeinfo.tm_hour, timeinfo.tm_min, timeinfo.tm_sec,
            timeinfo.tm_year + 1900, timeinfo.tm_mon + 1, timeinfo.tm_mday);


  (*syncEventHandler)();

  vTaskDelete(NULL);
}


namespace SpiderLib {

  SNTP::SNTP() {

    const char* server = "pool.ntp.org";  // TODO: make configurable!

    ESP_LOGI(TAG, "Starting synchronization against server[ %s ]", server);

    sntp_setoperatingmode(SNTP_OPMODE_POLL);
    sntp_setservername(0, server);
    sntp_init();

    setenv("TZ", "CET-1CEST,M3.5.0,M10.5.0/3", 1);  // germany
    tzset();

    xTaskCreate(ntpTask, TAG, 8192, (void*)&syncEventHandler, 0, NULL);
  }

  void SNTP::registerTimeSyncHandler(std::function<void()> f) {
    if (inSync()) {
      f();
    } else {
      std::function<void()> _f = syncEventHandler;
      syncEventHandler = [=]() {
        _f();
        f();
      };
    }
  }

  bool SNTP::inSync() {
    time_t now = 0;
    struct tm timeinfo;
    memset(&timeinfo, 0, sizeof(tm));

    time(&now);
    localtime_r(&now, &timeinfo);

    return timeinfo.tm_year > 100;
  }



}
